﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace RX7Interface
{
    public partial class Gauge : AquaControls.AquaGauge
    {
        private float minValue;
        private float maxValue;

        private Parameter parameter;

        public bool AutoAdaptLimits
        {
            get;
            set;
        }

        public Gauge()
        {
            InitializeComponent();

            AutoAdaptLimits = false;
        }

        public void ShowGaugeParameter(Parameter parameter)
        {
            if (!parameter.Enabled)
            {
                return;
            }

            this.parameter = parameter;

            // Reset the limits, as the AqueGauge behaves oddly if
            // setting a MinValue that is greater than MaxValue and vice versa.
            this.MinValue = float.MinValue;
            this.MaxValue = float.MaxValue;

            // Setup our locally stored limits so they will be set as soon as the first
            // data value arrives.
            minValue = float.MaxValue;
            maxValue = float.MinValue;

            if (parameter.GaugeInfo == null)
            {
                AutoAdaptLimits = true;

                this.Value = 0;
                this.RecommendedValue = 0;
                this.ThresholdPercent = 20;
            }
            else
            {
                AutoAdaptLimits = false;

                this.MinValue = (float)parameter.GaugeInfo.MinValue;
                this.MaxValue = (float)parameter.GaugeInfo.MaxValue;
                this.RecommendedValue = parameter.GaugeInfo.ReccomendedValue;
                this.ThresholdPercent = parameter.GaugeInfo.ReccomendedValueRange;
            }

            this.DialText = parameter.Name + " (" + parameter.Units + ")";

            this.Update();
        }

        private float RoundToSignicantFigure(float value, int significantFigures, bool roundUp)
        {
            if (roundUp)
            {
                value *= 1.2f;
            }
            else
            {
                value *= 0.8f;
            }

            int power = 0;

            while (value > 10)
            {
                value /= 10;
                power++;
            }

            value = (float)Math.Round((double)value, significantFigures - 1);

            value *= (float)Math.Pow(10, power);

            return value;
        }

        public new void Update()
        {
            if (this.parameter != null)
            {
                float value = (float)parameter.DataValue.GetValue();

                if (AutoAdaptLimits)
                {
                    if (minValue > value)
                    {
                        minValue = value;
                        this.MinValue = RoundToSignicantFigure(value, 2, false);
                        this.RecommendedValue = (this.MinValue + this.MaxValue) / 2;
                    }

                    if (value > maxValue)
                    {
                        maxValue = value;
                        this.MaxValue = RoundToSignicantFigure(value, 2, true);
                        this.RecommendedValue = (this.MinValue + this.MaxValue) / 2;
                    }
                }

                this.Value = value;
            }

            base.Invalidate();
        }
    }
}
